/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Modular D Engine
 *
 * This module contains main(), which calls Init and runs the main loop.
 */
module mde.mde;

// Comment to show use, where only used minimally:

import global = mde.global;             // global.run
import gl = mde.gl.draw;                // gl.draw()
import mde.events;                      // pollEvents

import mde.scheduler.Init;
import mde.scheduler.Scheduler;         // Scheduler.run()
import mde.scheduler.exception;         // InitException

import tango.core.Thread : Thread;	// Thread.sleep()
import tango.time.Clock;                // Clock.now()
import tango.util.log.Log : Log, Logger;

int main()
{
    //BEGIN Initialisation
    Logger logger = Log.getLogger ("mde.mde");
    logger.info ("Starting mde...");
    
    scope Init init;
    try {
        init = new Init();	// initialisation
    } catch (InitException e) {
        logger.fatal ("Initialisation failed: " ~ e.msg);
        return 1;
    }
    //END Initialisation
    
    //BEGIN Main loop setup
    Scheduler scheduler = new Scheduler;    // main loop's scheduler
    
    scheduler.add (scheduler.getNewID, &mde.events.pollEvents).frame = true;
    scheduler.add (global.SCHEDULE.DRAW, &gl.draw).frame = true;    // draw. for now, every frame.
    //END Main loop setup
    
    while (global.run) {
        scheduler.execute (Clock.now());
        
        Thread.sleep (0.050);	// sleep this many seconds
    }
    
    return 0;		// cleanup handled by init's DTOR
}
